﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO.Ports;

namespace WindowsFormsApplication1
{
    public partial class GUI : Form
    {
        /*Constant variables*/
        const int SWEEP = 0;
        const int SINGLE = 1;

        /*Thread related variables*/
        Thread spectrumUpdateThread;
        bool spectrumAnalyserThreadRunning;
        bool spectrumAnalyserMutex; //Access control to device 

        /*Delegate method fot invoking chart*/
        delegate void drawChartInvoker();

        /*Used to sync device datastream*/
        bool framed = false;

        /*UART port object*/
        SerialPort port;

        /*Frequency parameters and storage*/
        int startFrequency;
        int endFrequency;
        int stepFrequency;
        int currentFrequency;
        double[] singleFrequencyList;

        /*Mode select variable initialized to frequency sweep mode*/
        int mode = SWEEP;

        public GUI()
        {
            /*Initialize control component*/
            InitializeComponent();

            /*Initialize communication to device*/
            InitializeUART();

            /*Initialize device through communication channel*/
            InitializeSpectrumAnalyser();

            /*Initialize frequency parameters from GUI controls*/
            startFrequency = (int)(numericUpDownCentreFreq.Value - numericUpDownSpanFreq.Value/2);
            endFrequency = (int)(numericUpDownCentreFreq.Value + numericUpDownSpanFreq.Value / 2);
            stepFrequency = (int)numericUpDownFreqStep.Value;
            currentFrequency = startFrequency;

            /*Initialize chart components*/
            spectrumChart.Series[0].Name = "Instant";
            spectrumChart.Series.Add("Maximum");
            spectrumChart.Series.Add("Minimum");
            
            /*Initialize Spectrum analyser graph*/
            InitializeSpectrumChart(); 
            
            /*Initialize spectrum analyser thread control*/
            spectrumAnalyserMutex = false;

            //Initialize form controls
            listBoxSingleFreq.Items.Remove(0);
            buttonSingleFreqRemove.Enabled = false;
           
        }

        /*Initialize the spectrum analyser device by first searching
         for a connection and the starting its update thread*/
        private void InitializeSpectrumAnalyser()
        {
            /*Configure device*/
            string command = getRangeCommand();
            checkOutMutex(ref spectrumAnalyserMutex);
            sendUARTString(command);
            returnMutex(ref spectrumAnalyserMutex);
            
            /*Start spectrum update thread*/
            spectrumUpdateThread = new Thread(spectrumAnalyserUpdateThread);
            spectrumUpdateThread.Start();
        }

        private string getRangeCommand()
        {
            string command = "R" + startFrequency + endFrequency; //Define command
            //Add padding in front of step size if needed
            string stepSize = stepFrequency + "";
            while (stepSize.Length < 6)
                stepSize = "0" + stepSize;
            command = command + stepSize + ".";

            return command;
        }

        /*Initialize Charts to there default startup states.*/
        private void InitializeSpectrumChart()
        {
            /*Clear Graph*/
            spectrumChart.Series["Instant"].Points.Clear();
         
            /*Set labels*/
            spectrumChart.ChartAreas[0].AxisX.Title = "Frequency [MHz]";
            spectrumChart.ChartAreas[0].AxisY.Title = "Receive Level [dBm]";

            /*Configure for sweep mode*/
            if (mode == SWEEP)
            {
                /*Set graph to a line graph*/
                spectrumChart.Series["Instant"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.FastLine;
                spectrumChart.Series["Maximum"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.FastLine;
                spectrumChart.Series["Minimum"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.FastLine;

                /*Set X axis ranges*/
                spectrumChart.ChartAreas[0].AxisX.Minimum = (((double)startFrequency) / 1000);
                spectrumChart.ChartAreas[0].AxisX.Maximum = (((double)endFrequency) / 1000);
                spectrumChart.ChartAreas[0].AxisX.Interval = ((((double)startFrequency - (double)endFrequency) / 1000) / 20);
                
                /*Set Y axis ranges*/
                spectrumChart.ChartAreas[0].AxisY.Minimum = -120;
                spectrumChart.ChartAreas[0].AxisY.Maximum = -60;
                spectrumChart.ChartAreas[0].AxisY.Interval = 10;

            }
            /*Set graph for single frequency mode*/
            else if (mode == SINGLE)
            {
                /*Set to column graph*/
                spectrumChart.Series["Instant"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.RangeColumn;
                spectrumChart.Series["Maximum"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.RangeColumn;
                spectrumChart.Series["Minimum"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.RangeColumn;

                /*Set X axis ranges*/
                spectrumChart.ChartAreas[0].AxisX.Minimum = 240;
                spectrumChart.ChartAreas[0].AxisX.Maximum = 960;
                spectrumChart.ChartAreas[0].AxisX.Interval = 40;

                /*Set Y axis ranges*/
                spectrumChart.ChartAreas[0].AxisY.Minimum = 0;
                spectrumChart.ChartAreas[0].AxisY.Maximum = 80;
                spectrumChart.ChartAreas[0].AxisY.Interval = 10;
            }
           
            /*Set additional graph properties*/
            spectrumChart.ChartAreas[0].AxisY.MajorGrid.LineColor = Color.LightGray;
            spectrumChart.ChartAreas[0].AxisX.MajorGrid.LineColor = Color.LightGray;
        }

        /*This thread handles the spectrum analyser update cycle
         by ensuring the data is framed and then calling the graph 
         draw when neccesary*/
        private void spectrumAnalyserUpdateThread()
        {
            Thread.Sleep(1000);
            spectrumAnalyserThreadRunning = true;
            while (spectrumAnalyserThreadRunning)
            {
               // checkOutMutex(ref spectrumAnalyserMutex);
                if (mode == SWEEP && framed) //Wait for framing bits
                {
                    /*Read RS232 and populate chartArray*/
                    
                    /*Draw information to charts*/
                    try
                    {
                        spectrumChart.Invoke(new drawChartInvoker(updateGraph));
                    }
                    catch (InvalidOperationException e)
                    {

                    }



                    currentFrequency += stepFrequency;
                    if (currentFrequency >= endFrequency)
                    {
                        currentFrequency = startFrequency;

                        /*Check if data is still framed*/
                        //Check for framing bits
                        int frameBitNo = 0;
                        while (getSignalStrength() == -120)
                            frameBitNo++;
                        if (frameBitNo != 3)
                        {
                            framed = false;
                            //MessageBox.Show("Framing lost");
                        }

                    }
                }
                else if(mode == SWEEP)
                {
                    //Check for framing bits
                    int frameBitNo = 0;
                    while (getSignalStrength() == -120)
                    {
                        frameBitNo++;
                    }
                    if (frameBitNo == 3)
                    {
                        framed = true;
                        currentFrequency = startFrequency;
                    }
                  
                }
                
                if(mode == SINGLE)
                {
                    /*Check if list is still valid*/
                    if(singleFrequencyList == null || singleFrequencyList.Length != listBoxSingleFreq.Items.Count)
                    {
                        singleFrequencyList = new double[listBoxSingleFreq.Items.Count];
                    }

                    for (int i = 0; i < singleFrequencyList.Length; i++)
                    {
                        int frequency = Int32.Parse(listBoxSingleFreq.Items[i].ToString());
                        singleFrequencyList[i] = getSignalStrength(frequency) + 120;
                    }

                    /*Draw information to charts*/
                    try
                    {
                        spectrumChart.Invoke(new drawChartInvoker(updateGraph));
                    }
                    catch (InvalidOperationException e)
                    {

                    }
                }
               //returnMutex(ref spectrumAnalyserMutex);
            }
        }

        private void updateGraph()
        {
            try
            {
               if(mode == SWEEP)
               {
                    if (spectrumChart.Series["Instant"].Points.Count <= ((int)(currentFrequency - startFrequency) / stepFrequency))
                        spectrumChart.Series["Instant"].Points.AddXY((double)currentFrequency / 1000, getSignalStrength());
                    else
                    {
                        spectrumChart.Series["Instant"].Points.RemoveAt(((int)(currentFrequency - startFrequency) / stepFrequency));
                        spectrumChart.Series["Instant"].Points.InsertXY(((int)(currentFrequency - startFrequency) / stepFrequency), (double)currentFrequency / 1000, getSignalStrength());
                    }
                }
               else if (mode == SINGLE)
               {
                 
                   spectrumChart.Series["Instant"].Points.Clear();
                   
                   if(singleFrequencyList != null)
                   for (int i = 0; i < singleFrequencyList.Length; i++)
                   {
                       int frequency = Int32.Parse( listBoxSingleFreq.Items[i].ToString());
                      
                       spectrumChart.Series["Instant"].Points.AddXY(frequency / 1000, singleFrequencyList[i]);
                   }
                   
               }
               spectrumChart.Update();
            }
            catch (ArgumentOutOfRangeException e)
            {

            }
           
        }

        private double getSignalStrength()
        {
            //Get value from buffer
            double signal = readUARTByte();
            
            //Check for framing
            //TODO

            signal = (signal*0.471 - 120);

            return signal;
        }
        private double getSignalStrength(int frequency)
        {
            //First set frequency
            string command = "S" + frequency + "."; //Define command
            checkOutMutex(ref spectrumAnalyserMutex);
            sendUARTString(command);
            returnMutex(ref spectrumAnalyserMutex);
          

            Thread.Sleep(100); //Wait for frequency to settle

            //Get value from buffer

            double signal = 0;
            for (int i = 0; i < 50; i++)
            {
                signal += readUARTByte();
                Thread.Sleep(1);
            }
            signal = signal / 50;

            signal = (signal * 0.471 - 120);

            return signal;
        }
        /*Thread access control methods*/
        private void checkOutMutex(ref bool mutex)
        {
            while (mutex) { }//Wait for mutex to be released
            mutex = true; //Make resource unavailable
        }
        private void returnMutex(ref bool mutex)
        {
            mutex = false; //Make resource available
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }
        private void label3_Click(object sender, EventArgs e)
        {

        }

        private void numericUpDown4_ValueChanged(object sender, EventArgs e)
        {

        }

        private void buttonSingleFreqAdd_Click(object sender, EventArgs e)
        {
            /*First check if frequency does not already exist*/
            if (!listBoxSingleFreq.Items.Contains(numericUpDownSingleFreq.Value))
                  listBoxSingleFreq.Items.Add(numericUpDownSingleFreq.Value);

            /*Enable remove button*/
            buttonSingleFreqRemove.Enabled = true;
        }

        private void buttonSingleFreqRemove_Click(object sender, EventArgs e)
        {
            /*Remove selected item from list*/
            listBoxSingleFreq.Items.Remove(listBoxSingleFreq.SelectedItem);

            /*Disable remove button when list is empty*/
            if (listBoxSingleFreq.Items.Count == 0)
            {
                buttonSingleFreqRemove.Enabled = false;
            }
     
        }

        private void InitializeUART()
        {
            // Instantiate the communications
            // port with some basic settings 
            port = new SerialPort("COM11", 9600, Parity.None, 8, StopBits.One);

            // Open the port for communications 
            port.Open(); 
            
        }

        // that contains the SerialPort class  
        private int readUARTByte() 
        { 
            int data;
            try
            {
                // Write a string 
               data = port.ReadByte();
            }
            catch(System.IO.IOException e)
            {
                data = -1;
            }
            return data;
   
        }
        // that contains the SerialPort class  
        private void sendUARTString(string text)
        {

            // Write a string 
            //MessageBox.Show(text);
            for (int i = 0; i < text.Length; i++)
            {
                port.Write(text.ToCharArray()[i] + "");
                Thread.Sleep(1);
            }


        }

        private void GUI_FormClosed(object sender, FormClosedEventArgs e)
        {
            //MessageBox.Show("Baai");
            spectrumAnalyserThreadRunning = false;
            // Close the port 
            port.Close(); 
        }

        private void buttonSetRange_Click(object sender, EventArgs e)
        {
            startFrequency = (int)(numericUpDownCentreFreq.Value - numericUpDownSpanFreq.Value / 2);
            endFrequency = (int)(numericUpDownCentreFreq.Value + numericUpDownSpanFreq.Value / 2);
            stepFrequency = (int)numericUpDownFreqStep.Value;

            //checkOutMutex(ref spectrumAnalyserMutex);
            string command = "R" + startFrequency + endFrequency; //Define command
            //Add padding in front of step size if needed
            string stepSize =  stepFrequency+ "";
            while (stepSize.Length < 6)
                stepSize = "0" + stepSize;
            command = command + stepSize + ".";
            spectrumChart.Invoke(new drawChartInvoker(InitializeSpectrumChart));

            currentFrequency = startFrequency;
            framed = false;
            //Thread.Sleep(100);
            //MessageBox.Show(command);
            checkOutMutex(ref spectrumAnalyserMutex);
            sendUARTString(command);
            returnMutex(ref spectrumAnalyserMutex);
           
            //returnMutex(ref spectrumAnalyserMutex);
        }

        private void radioButtonSweepFreqMode_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButtonSingleFreqMode.Checked)
            {
                mode = SINGLE;
                spectrumChart.Invoke(new drawChartInvoker(InitializeSpectrumChart));
            }
            else if (radioButtonSweepFreqMode.Checked)
            {
                mode = SWEEP;
                startFrequency = (int)(numericUpDownCentreFreq.Value - numericUpDownSpanFreq.Value / 2);
                endFrequency = (int)(numericUpDownCentreFreq.Value + numericUpDownSpanFreq.Value / 2);
                stepFrequency = (int)numericUpDownFreqStep.Value;

                //checkOutMutex(ref spectrumAnalyserMutex);
                string command = "R" + startFrequency + endFrequency; //Define command
                //Add padding in front of step size if needed
                string stepSize = stepFrequency + "";
                while (stepSize.Length < 6)
                    stepSize = "0" + stepSize;
                command = command + stepSize + ".";
                spectrumChart.Invoke(new drawChartInvoker(InitializeSpectrumChart));

                currentFrequency = startFrequency;
                framed = false;
                //Thread.Sleep(100);
                //MessageBox.Show(command);
                checkOutMutex(ref spectrumAnalyserMutex);
                sendUARTString(command);
                returnMutex(ref spectrumAnalyserMutex);


            }
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            string command = "G" + ((char)numericUpDown1.Value)+".";
            checkOutMutex(ref spectrumAnalyserMutex);
            sendUARTString(command);
            returnMutex(ref spectrumAnalyserMutex);
        }

    }
}
